//
//  sce.cpp
//  PIC
//
//  Created by Fei Gao on 4/23/15.
//  Copyright (c) 2015 Fei Gao. All rights reserved.
//
// This code is used to solve for NPMLE of F for PIC data using self-consistency algorithm
// Input:  VectorXd L,R,censor, censori, Z
// Output: t and F

#include <Rcpp.h>
#include "PIC_AFT.h"
#include <iostream>
#include <fstream>
#include <unistd.h>
#include "/Users/rita-gaofei/Documents/eigen-eigen-10219c95fe65/Eigen/Dense"
using namespace Eigen;
using namespace std;
using namespace Rcpp;

// [[Rcpp::export]]
NumericMatrix PIC_BJ(NumericVector L, NumericVector R, NumericMatrix X, IntegerVector censor, IntegerVector censori,int nboot){
    int n = X.nrow(); int p = X.ncol();
    VectorXd LL(n); VectorXd RR(n); VectorXi censorr(n); VectorXi censorii(n); MatrixXd XX(n,p);
    for (int i=0;i<n;++i){
        LL(i) = L[i]; RR(i) = R[i]; censorr(i) = censor[i]; censorii(i) = censori[i];
        for (int j=0;j<p;++j) XX(i,j) = X(i,j);
    }
    AFT_PIC data(LL, RR, XX, censorr, censorii);
    if (nboot==0){
        NumericMatrix res(p,1);
        data.BJ_est();
        for (int j=0;j<p;++j) res(j,0) = data.beta_(j);
        return(res);
    } else {
        NumericMatrix res(p,2);
        data.BJ_est();
        for (int j=0;j<p;++j) res(j,0) = data.beta_(j);
        default_random_engine generator;
        data.BJ_boot(nboot,generator);
        for (int j=0;j<p;++j) res(j,1) = data.betaSE_(j);
        return(res);
    }
}
// [[Rcpp::export]]
NumericMatrix PIC_onestep(NumericVector L, NumericVector R, NumericMatrix X, IntegerVector censor, IntegerVector censori,int nboot){
    int n = X.nrow(); int p = X.ncol();
    VectorXd LL(n); VectorXd RR(n); VectorXi censorr(n); VectorXi censorii(n); MatrixXd XX(n,p);
    for (int i=0;i<n;++i){
        LL(i) = L[i]; RR(i) = R[i]; censorr(i) = censor[i]; censorii(i) = censori[i];
        for (int j=0;j<p;++j) XX(i,j) = X(i,j);
    }
    AFT_PIC data(LL, RR, XX, censorr, censorii);
    if (nboot==0){
        NumericMatrix res(p,1);
        data.Onestep_est();
        for (int j=0;j<p;++j) res(j,0) = data.betaeff_(j);
        return(res);
    } else {
        NumericMatrix res(p,2);
        data.Onestep_est();
        for (int j=0;j<p;++j) res(j,0) = data.betaeff_(j);
        default_random_engine generator;
        data.Onestep_boot(nboot,generator);
        for (int j=0;j<p;++j) res(j,1) = data.betaeffSE_(j);
        return(res);
    }
}
