//
//  sce.h
//  PIC
//
//  Created by Fei Gao on 4/24/15.
//  Copyright (c) 2015 Fei Gao. All rights reserved.
// class sce
// Input:L, R, censor, censori, Z
// censori: left-1,interval-2, right-3
// If censored, t in (L,R), If censori=1 then L=-infty; if censori=3 then R=infty

#ifndef __PIC__sce__
#define __PIC__sce__

#include <stdio.h>
#include <iostream>
#include "/Users/rita-gaofei/Documents/eigen-eigen-10219c95fe65/Eigen/Dense"
using namespace Eigen;
using namespace std;


VectorXd VecSort(VectorXd x); // Sort VectorXd increasingly
VectorXd VecSortUniq(VectorXd x); // Sort VectorXd increasingly and with only unique values
class sce
{
private:
    long n_;
    long njump_;
    double nZ_;
    VectorXd L_; VectorXd R_;
    VectorXi censor_; VectorXi censori_;// 1 for left, 2 for interval, 3 for right
    VectorXd Z_; VectorXd F0_;
    double FL_;double FR_;double FLt_;double FRt_;
    double dnormF_;
    const double epsilon=1E-8;
    const int maxiter=1000;
    int iter;
    
    // Initiate the Matrix for F, jump only at exactly observed time
    void tinit()
    {   // all the exact failure times
        int d=censor_.sum();
        VectorXd Lt(d);
        long line=0;
        for (int i=0;i<n_;++i)
        {if (censor_(i)==1) {Lt(line)=L_(i); line=line+1;}}
        t_=VecSortUniq(Lt);
        njump_=t_.size();
    }
    void Finit()
    {   F0_.resize(njump_);
        F0_.setZero();
        F_.resize(njump_);
        for (int i=0;i<njump_;++i)
        {   F_(i)=((double)i+1)/((double)njump_+1);
        }
    }
    
    void findLR()
    {   //Initital Lline and Rline
        Lline_.resize(n_);Rline_.resize(n_);
        for (int i=0; i<n_;++i)
        { int t=0; while (t<njump_){if (L_(i)>=t_(t)) {t=t+1;}else break;} Lline_(i)=t-1;
            t=0; while (t<njump_){if (R_(i)>=t_(t)) {t=t+1;}else break;} Rline_(i)=t-1;
        }
        
    }
    
    void dF()
    {   VectorXd Fdiff=F_-F0_;
        dnormF_=Fdiff.norm()/F_.norm();
    }
    
    //Update F at new time points to compare
    VectorXd updateF(VectorXd t0, VectorXd F0, VectorXd t)
    {   long njump0=t0.size();
        long njump=t.size();
        long tj;
        VectorXd F;
        F.resize(njump);
        for (int i=0;i<njump;++i)
        {   tj=0;
            while (tj<njump0)
            {   if (t(i)>=t0(tj)) tj=tj+1;
            else break;
            }
            if (tj>0) F(i)=F0(tj-1);else F(i)=0;
        }
        return(F);
    }
    
public:
    VectorXd t_;VectorXd F_;
    VectorXi Lline_;VectorXi Rline_;
    bool cov_;
    
    sce() = default;
    sce(VectorXd L,VectorXd R, VectorXi censor, VectorXi censori,VectorXd Z)//no initial value for F
    {
        n_=L.rows();
        L_=L;R_=R;censor_=censor; censori_=censori;Z_=Z;
        nZ_=Z_.sum();
        tinit();Finit();
        findLR();
    }
    sce(VectorXd L,VectorXd R, VectorXi censor, VectorXi censori,VectorXd Z,VectorXd t0,VectorXd F0)
    {
        n_=L.rows();
        L_=L;R_=R;censor_=censor; censori_=censori;Z_=Z;
        nZ_=Z_.sum();
        tinit();
        F_=updateF(t0, F0, t_);
        findLR();
    }
    void fonestep()
    {   F0_=F_;F_.setZero();
        for (int t=0; t<njump_;++t)
        {   for (int i=0; i<n_;++i)
            {   if (censor_(i)==1){if (Lline_(i)<=t){F_(t)=F_(t)+Z_(i)/nZ_;}}
                else
                {   if (Lline_(i)<0){FL_=0;FLt_=0;}
                    else
                    {   FL_=F0_(Lline_(i));
                        if (Lline_(i)<t) {FLt_=F0_(Lline_(i));}
                        else {FLt_=F0_(t);}
                    }
                    if (Rline_(i)<0){FR_=0;FRt_=0;}
                    else
                    {   FR_=F0_(Rline_(i));
                        if (Rline_(i)<t) {FRt_=F0_(Rline_(i));}
                        else {FRt_=F0_(t);}
                    }
                    if (censori_(i)==1) {FL_=0;FLt_=0;}
                    else if (censori_(i)==3){FR_=1;FRt_=F0_(t);}
                    if (FR_>FL_){F_(t)=F_(t)+Z_(i)*(FRt_-FLt_)/(FR_-FL_)/nZ_;}
                }
            }
        }
    }
    void fsolve()
    {   dnormF_=1;iter=0;
        while ((dnormF_>=epsilon)&(iter<maxiter)){fonestep();dF();}
        cov_=(iter<maxiter);
        /*cout << "Jump of Cumulative Density Function" << endl;
        cout << t_<< endl;*/
        if (!cov_) {cout<< "The self-consistency algorithm didn't converge in "<<maxiter<<" iterations!"<<endl;}
        //else {cout<<"The NPMLE for F is"<<F_<<endl;}
    }
};



// Sort VectorXd increasingly and with only unique values
VectorXd VecSortUniq(VectorXd x){
    long p=x.size(); double x1;
    for (int i=1;i<p;++i){
        for(int j=0;j<p-i;++j){
            if (x(j)>=x(j+1)){
                x1=x(j); x(j)=x(j+1);x(j+1)=x1;
            }
        }
    }
    VectorXd z(1); z(0)=x(0); int linez=0;
    for(int i=1;i<p;++i){
        if (x(i)>z(linez)){
            VectorXd z1=z; linez=linez+1; z.resize(linez+1);
            for (int i=0;i<linez;++i) z(i)=z1(i);
            z(linez)=x(i);
        }
    }
    return(z);
}

VectorXd VecSort(VectorXd x){
    long p=x.size(); double x1;
    for (int i=1;i<p;++i){
        for(int j=0;j<p-i;++j){
            if (x(j)>=x(j+1)){x1=x(j);x(j)=x(j+1);x(j+1)=x1;}
        }
    }
    return(x);
}

#endif /* defined(__PIC__sce__) */
